Imports System.Drawing.Drawing2D
''' <summary>
''' Control zum Darstellen einer Bitmap in optimaler Vergrerung
''' </summary>

Public Class BitmapPainter
    Inherits Control

#Region " Private Felder "

    ' darzustellende Bitmap
    Private m_Bitmap As Bitmap
    ' Rechteck zum Zeichnen der Bitmap
    Private m_BitmapRect As RectangleF

    ' Pen mit heller Rahmenfarbe
    Private m_LightLinePen As Pen
    ' Pen mit dunkler Rahmenfarbe
    Private m_DarkLinePen As Pen
    ' Hintergrundfarbe
    Private m_BackColor As Color

    ' Renderer, der Farben fr das Control zur Verfgung stellt
    Private WithEvents m_Renderer As ColorWheelToolStripRenderer

#End Region

#Region " Konstruktoren "

    ''' <summary>
    ''' Konstruktor fr einen neuen BitmapPainter
    ''' </summary>
    Public Sub New()
        ' neuen Renderer erstellen 
        m_Renderer = New ColorWheelToolStripRenderer(New ColorWheelProfessionalTable)
        m_Renderer.ControlHue = 0.4R

        ' DoubleBuffering aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint _
        Or ControlStyles.AllPaintingInWmPaint, True)

        ' Pens und Farben vom Renderer bernehmen
        GetPensAndColors()
        ' Bitmap-Rechteck aktualisieren
        GetBitmapRect()
    End Sub

#End Region

#Region " Properties "

    ''' <summary>
    ''' gibt den Renderer, der Farben fr das Control zur Verfgung stellt, zurck oder legt ihn fest
    ''' </summary>
    Property Renderer() As ColorWheelToolStripRenderer
        Get
            Return m_Renderer
        End Get
        Set(ByVal value As ColorWheelToolStripRenderer)
            ' nur wenn tatschlich ein neuer Renderer bergeben wird
            If value IsNot Nothing AndAlso m_Renderer IsNot value Then
                ' Renderer bernehmen
                m_Renderer = value
                ' Pens und Farben neu festlegen
                GetPensAndColors()
                ' Control neu zeichnen
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' legt die zu zeichnende Bitmap neu fest
    ''' </summary>
    ''' <value></value>
    WriteOnly Property Bitmap() As Bitmap
        Set(ByVal value As Bitmap)
            If m_Bitmap IsNot value Then
                m_Bitmap = value
                ' Bitmap-Rechteck aktualisieren
                GetBitmapRect()
                ' Control neu zeichnen
                Invalidate()
            End If
        End Set
    End Property

#End Region

#Region " Methoden "

    ''' <summary>
    ''' Handler fr das Changed-Event des Renderers
    ''' </summary>
    Private Sub m_Renderer_Changed() Handles m_Renderer.Changed
        ' Pens und Farben neu festlegen
        GetPensAndColors()
        ' Control neu zeichnen
        Invalidate()
    End Sub

    ''' <summary>
    ''' bernimmt Pens und Farben vom Renderer
    ''' </summary>
    Private Sub GetPensAndColors()
        ' heller Pen zum Zeichnen von Linien
        m_LightLinePen = m_Renderer.SeparatorLightPen
        ' dunkler Pen zum Zeichnen von Linien
        m_DarkLinePen = m_Renderer.SeparatorDarkPen
        ' Hintergrundfarbe
        m_BackColor = m_Renderer.ControlBackColor
    End Sub

    ''' <summary>
    ''' berschreiben der OnPaint-Methode zum Neuzeichnen des Controls
    ''' </summary>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)
        ' Referenz zum Graphics-Objekt
        Dim g As Graphics = e.Graphics

        ' Flche mit Hintergrundfarbe fllen
        g.Clear(m_BackColor)

        ' Wenn das Control zum Zeichnen der Bitmap zu klein ist, nichts weiteres tun
        If m_BitmapRect.Width < 1 OrElse m_BitmapRect.Height < 1 Then Exit Sub


        ' nur wenn eine Bitmap zum Zeichnen existiert
        If m_Bitmap IsNot Nothing Then

            ' bisherigen PixelOffsetMode und InterpolationMode merken
            Dim POM As Drawing2D.PixelOffsetMode = g.PixelOffsetMode
            Dim IM As Drawing2D.InterpolationMode = g.InterpolationMode

            ' PixelOffsetMode und InterpolationMode auf geeignete Werte umstellen,
            ' um auch fr Bitmaps mit sehr kleiner Kantenlnge eine unverwischte
            ' Darstellung zu erreichen
            g.PixelOffsetMode = Drawing2D.PixelOffsetMode.HighQuality
            g.InterpolationMode = Drawing2D.InterpolationMode.NearestNeighbor

            ' Bitmap zeichnen
            g.DrawImage(m_Bitmap, m_BitmapRect)

            ' frhere Werte fr PixelOffsetMode und InterpolationMode bernehmen
            g.PixelOffsetMode = POM
            g.InterpolationMode = IM

        End If

        ' helle Linie am Unterrand des Controls zeichnen
        g.DrawLine(m_LightLinePen, 0, Me.Height - 1, Me.Width, Me.Height - 1)

        ' Bildrahmen zeichnen
        g.DrawLine(m_LightLinePen, m_BitmapRect.Left, m_BitmapRect.Bottom, m_BitmapRect.Right, m_BitmapRect.Bottom)
        g.DrawLine(m_LightLinePen, m_BitmapRect.Right, m_BitmapRect.Top, m_BitmapRect.Right, m_BitmapRect.Bottom)
        g.DrawLine(m_DarkLinePen, m_BitmapRect.Left, m_BitmapRect.Top, m_BitmapRect.Right, m_BitmapRect.Top)
        g.DrawLine(m_DarkLinePen, m_BitmapRect.Left, m_BitmapRect.Top, m_BitmapRect.Left, m_BitmapRect.Bottom)

    End Sub

    ''' <summary>
    ''' berschreiben der OnResize-Methode des Controls
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        ' Rechteck zum Zeichnen der Bitmap neu ermitteln
        GetBitmapRect()
        ' Control neu zeichnen
        Invalidate()
    End Sub

    ''' <summary>
    ''' berechnet das Rechteck zum Zeichnen der Bitmap / des Bildrahmens neu
    ''' </summary>
    Private Sub GetBitmapRect()
        If Me.Width < 33 OrElse Me.Height < 33 Then
            ' Wenn das Control zu klein zum Anzeigen einer Bitmap ist, erhlt das 
            ' Anzeigerechteck als Kantenlnge 0
            m_BitmapRect = New RectangleF(0, 0, 0, 0)
        ElseIf m_Bitmap Is Nothing Then
            ' Wenn keine Bitmap zum Zeichnen vorhanden ist, wird ein Quadrat erstellt
            Dim MinWidth As Single = Math.Min(Me.Width, Me.Height)
            MinWidth -= 32.0F

            Dim x As Single = CSng(Me.Width - MinWidth) / 2.0F
            Dim y As Single = CSng(Me.Height - MinWidth) / 2.0F
            m_BitmapRect = New RectangleF(x, y, MinWidth, MinWidth)
        Else
            ' Wenn eine Bitmap zum Zeichnen vorhanden ist, wird ein Rechteck mit 
            ' skalierter Kantenlnge erstellt
            Dim pw As Single = m_Bitmap.Width
            Dim ph As Single = m_Bitmap.Height

            Dim dw As Single = Me.Width - 32.0F
            Dim dh As Single = Me.Height - 32.0F

            Dim scaleFactor As Single
            If pw / ph > dw / dh Then
                scaleFactor = dw / pw
            Else
                scaleFactor = dh / ph
            End If

            Dim Width As Single = scaleFactor * pw
            Dim Height As Single = scaleFactor * ph
            Dim x As Single = (dw - Width) / 2.0F + 16.0F
            Dim y As Single = (dh - Height) / 2.0F + 16.0F

            m_BitmapRect = New RectangleF(x, y, Width, Height)
        End If
    End Sub

#End Region

End Class
