Public Class UndoManager
    Private m_Stack As New List(Of Command)
    Private m_Position As Integer
    Private m_Maximum As Integer
    Private m_Gruppe As CommandGroup

    Public Event StateChanged As EventHandler


    Public Sub New(ByVal maxHistory As Integer)
        m_Stack = New List(Of Command)
        m_Maximum = maxHistory
    End Sub

    Public Sub Clear()
        If m_Stack.Count > 0 Then
            m_Position = 0
            m_Stack.Clear()
            RaiseEvent StateChanged(Me, EventArgs.Empty)
        End If
    End Sub

    Public ReadOnly Property CanUndo() As Boolean
        Get
            Return m_Position > 0
        End Get
    End Property

    Public ReadOnly Property CanRedo() As Boolean
        Get
            Return m_Position < m_Stack.Count
        End Get
    End Property

    Public ReadOnly Property Current() As Command
        Get
            If m_Position >= 0 AndAlso m_Position < m_Stack.Count Then
                Return m_Stack(m_Position)
            End If
            Return Nothing
        End Get
    End Property

    Private Sub Add(ByVal cmd As Command)
        If m_Position < m_Stack.Count Then
            m_Stack.RemoveRange(m_Position, m_Stack.Count - m_Position)
        End If
        System.Diagnostics.Trace.WriteLine(cmd.Name)
        m_Stack.Add(cmd)
        If m_Stack.Count > m_Maximum Then
            m_Stack.RemoveAt(0)
        Else
            m_Position += 1
        End If
    End Sub

    Public Sub Push(ByVal cmd As Command)
        If m_Gruppe IsNot Nothing Then
            m_Gruppe.Add(cmd)
        Else
            Add(cmd)
        End If

        ' Must do command after adding it to the command stack!
        cmd.RunFirst()
        RaiseEvent StateChanged(Me, EventArgs.Empty)

    End Sub

    Public Function Undo() As Command
        If m_Position > 0 Then
            m_Position -= 1
            Dim cmd As Command = Me.Current
            If cmd IsNot Nothing Then
                cmd.Undo()
                RaiseEvent StateChanged(Me, EventArgs.Empty)
            End If
            Return cmd
        End If
        Return Nothing
    End Function

    Public Function Redo() As Command
        If m_Position < m_Stack.Count Then
            Dim cmd As Command = Me.Current
            If cmd IsNot Nothing Then
                cmd.Redo()
                RaiseEvent StateChanged(Me, EventArgs.Empty)
            End If
            m_Position += 1
            Return cmd
        End If
        Return Nothing
    End Function

    Public Function Peek() As Command
        If m_Position > 0 AndAlso m_Position - 1 < m_Stack.Count Then
            Return m_Stack(m_Position - 1)
        End If
        Return Nothing
    End Function

    Public Function Pop() As Command
        ' remove command without undoing it.
        If m_Position > 0 Then
            m_Position -= 1
            If m_Position < m_Stack.Count Then
                m_Stack.RemoveAt(m_Position)
            End If
            RaiseEvent StateChanged(Me, EventArgs.Empty)
        End If
        Return Me.Current
    End Function

    Public Sub Merge(ByVal cmd As CommandGroup)
        ' Replace the current command without upsetting rest of the redo stack
        ' and insert the current command into this compound command.
        Dim current As Command = Peek()
        If current IsNot Nothing Then
            m_Stack.Insert(m_Position - 1, cmd)
            m_Stack.RemoveAt(m_Position)
            cmd.RunFirst()
            cmd.Insert(current)
        Else
            Push(cmd)
        End If
    End Sub

    Public Function OpenCompoundAction(ByVal name As String) As CommandGroup
        m_Gruppe = New CommandGroup(name)
        Add(m_Gruppe)
        Return m_Gruppe
    End Function

    Public Sub CloseCompoundAction()
        m_Gruppe = Nothing
    End Sub
End Class

Public MustInherit Class Command
    Public MustOverride ReadOnly Property Name() As String
    Public MustOverride Sub RunFirst()
    Public MustOverride Sub Undo()
    Public MustOverride Sub Redo()
End Class


Public Class CommandGroup
    Inherits Command
    Private m_commands As New List(Of Command)
    Private m_name As String

    Public Sub New(ByVal name As String)
        m_name = name
    End Sub

    Public Overloads Overrides ReadOnly Property Name() As String
        Get
            Return m_name
        End Get
    End Property

    Public Sub Add(ByVal cmd As Command)
        m_commands.Add(cmd)
    End Sub

    Public ReadOnly Property Count() As Integer
        Get
            Return m_commands.Count
        End Get
    End Property

    Public Overloads Overrides Sub RunFirst()
        For Each cmd As Command In m_commands
            cmd.RunFirst()
        Next
    End Sub
    Public Overloads Overrides Sub Undo()
        For i As Integer = m_commands.Count - 1 To 0 Step -1
            ' Must undo in reverse order!
            Dim cmd As Command = m_commands(i)
            cmd.Undo()
        Next
    End Sub
    Public Overloads Overrides Sub Redo()
        For Each cmd As Command In m_commands
            cmd.Redo()
        Next
    End Sub

    Public Sub Insert(ByVal cmd As Command)
        m_commands.Insert(0, cmd)
    End Sub
End Class
