'  Copyright (c) 2006, Gustavo Franco
'  Email:  gustavo_franco@hotmail.com
'  All rights reserved.

'  Redistribution and use in source and binary forms, with or without modification, 
'  are permitted provided that the following conditions are met:

'  Redistributions of source code must retain the above copyright notice, 
'  this list of conditions and the following disclaimer. 
'  Redistributions in binary form must reproduce the above copyright notice, 
'  this list of conditions and the following disclaimer in the documentation 
'  and/or other materials provided with the distribution. 

'  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
'  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
'  REMAINS UNCHANGED.

Imports Ctls
Imports System.IO
''' <summary>
''' Datei-Inhalt eines Icon inclusive Funktionen zum Bearbeiten
''' </summary>
Public Class IconDatei
    Inherits Datei

#Region " Felder "
    Private m_Images As IconImageList                   ' Liste mit Einzelbildern
    Private m_UndoManager As UndoManager                ' Aktionsverwalter fr Undo-Redo-Funktionalitt
    Private m_PersistedImages As New List(Of String)    ' Auflistung von Pfaden zu temporr gespeicherten IconImages
#End Region

#Region " Properties "
    ''' <summary>
    ''' Liste mit Einzelbildern des Icons
    ''' </summary>
    ''' <remarks>diese Liste sorgt selbstndig dafr, dass immer ein IconImage
    ''' als Selected gekennzeichnet ist</remarks>
    ReadOnly Property Images() As IconImageList
        Get
            Return m_Images
        End Get
    End Property

    ''' <summary>
    ''' Aktionsverwalter fr Undo-Redo-Funktionalitt
    ''' </summary>
    ReadOnly Property UndoManager() As UndoManager
        Get
            Return m_UndoManager
        End Get
    End Property
#End Region

#Region " Konstruktoren "
    ''' <summary>
    ''' Konstruktor zum Lesen einer Icon-Datei fr einen angegebenen Pfad
    ''' </summary>
    Public Sub New(ByVal Pfad As String)
        MyBase.New(Pfad)

        m_Images = New IconImageList
        m_UndoManager = New UndoManager(1000)

        ' (Wenn der Pfad leer ist, ist dies eine neue leere Datei)
        ' Wenn ein Pfad angegeben wurde, Inhalt der Datei lesen
        If Pfad <> "" Then
            ' Images aus Datei lesen und der Auflistung hinzufgen
            Dim Images As List(Of IconImage) = GetImages(Pfad)
            m_Images.AddRange(Images.ToArray)
        End If
    End Sub
#End Region

#Region " Methoden "

    ''' <summary>
    ''' liefert ein IconImage fr einen angegebenen temporren Pfad
    ''' </summary>
    ''' <param name="Path">temporrer Pfad</param>
    Public Function GetImage(ByVal Path As String) As IconImage
        If Path = "" Then Return Nothing
        ' Wenn ein IconImage mit dem angegebenen Pfad als Instanz in der
        ' IconImageList existiert, wird dieses zurck gegeben
        For Each Img As IconImage In m_Images
            If Img.TempName = Path Then
                Return Img
            End If
        Next

        ' Wenn kein IconImage mit dem angegebenen Pfad als Instanz in der
        ' IconImageList existiert, in den Pfaden fr temporr (im Temp-Verzeichnis) gespeicherten
        ' IconImages danach suchen - wenn vorhanden, dieses Bild neu lesen 
        ' und diese Instanz zurck geben
        For Each txt As String In m_PersistedImages
            If txt = Path Then
                Dim NewImage As IconImage = Me.GetImages(Path)(0)
                NewImage.TempName = Path
                Return NewImage
            End If
        Next
        Return Nothing
    End Function

    ''' <summary>
    ''' liefert alle in einer Icon-Datei enthaltenen IconImages als Auflistung
    ''' </summary>
    ''' <param name="filename">Pfad der Datei</param>
    Private Function GetImages(ByVal filename As String) As List(Of IconImage)
        ' Liste, die mit IconImages gefllt werden soll und spter
        ' zurck gegeben wird
        Dim Result As New List(Of IconImage)

        ' FileStream fr angegebenen Pfad erzeugen
        Dim fs As New FileStream(filename, FileMode.Open, FileAccess.Read)
        ' Binaryreader, der diese Datei liest
        Dim Reader As New BinaryReader(fs)
        Try
            ' IconHeader lesen
            Dim Head As New IconHeader(Reader)
            ' Anzahl Einzelbilder bestimmen
            Dim ImageCount As Integer = Head.ImageCount

            ' Array mit IconDirectories (fr jedes IconImage ein IconDirectory)
            Dim IconDirs(ImageCount - 1) As IconDirectory

            ' alle IconDirectories lesen
            For I As Integer = 0 To ImageCount - 1
                IconDirs(I) = New IconDirectory(Reader)
            Next

            ' fr jedes IconDirectory existieren Bilddaten fr ein IconImage
            ' wo sich diese Daten innerhalb der Datei befinden und wie viel Bytes
            ' diese Daten fr ein Icon Image lang sind, steht im IconDirectory
            For I As Integer = 0 To ImageCount - 1
                ' BytesInRes im IconDirectory gibt die Lnge der Bilddaten 
                ' fr ein IconImage in Bytes an

                ' also jetzt ein Array dieser Lnge fr die Bilddaten anlegen
                Dim imageDataBytes(IconDirs(I).BytesInRes - 1) As Byte
                ' ImageOffset im IconDirectory gibt die Position der Bilddaten in der Datei an
                fs.Seek(IconDirs(I).ImageOffset, SeekOrigin.Begin)
                ' das Bilddaten-Array durch Lesen des Streams fllen
                Reader.Read(imageDataBytes, 0, IconDirs(I).BytesInRes)

                ' die Art des IconImage (ob PNG oder Bitmap)
                ' ist in den ersten Bytes der Bilddaten enthalten
                Select Case imageDataBytes(0)
                    Case 40
                        ' BMP
                        Result.Add(New BitmapImage(IconDirs(I), imageDataBytes))
                    Case 137
                        If BitConverter.ToInt16(imageDataBytes, 1) = 20048 Then
                            ' PNG
                            Result.Add(New PNGImage(IconDirs(I), imageDataBytes))
                        End If
                End Select
            Next

        Finally
            ' Egal ob Fehler beim Lesen oder nicht, Reader und FileStream mssen
            ' geschlossen werden
            If Reader IsNot Nothing Then
                Reader.Close()
            End If
            If fs IsNot Nothing Then
                fs.Close()
            End If
        End Try
        Return Result
    End Function

    ''' <summary>
    ''' wird in der Basisklasse (Datei) bentigt, um festzustellen, ob der Inhalt leer ist
    ''' </summary>
    Public Overrides Function IsEmpty() As Boolean
        If m_Images.Count = 0 Then
            Return True
        End If
        Return False
    End Function

    ''' <summary>
    ''' Speichert die Datei unter einem angegebenen Pfad
    ''' </summary>
    Public Overrides Function Speichern(ByVal Pfad As String) As Boolean
        SaveImages(Pfad, m_Images)
        Return True
    End Function


    ''' <summary>
    ''' Speichert den Inhalt einer Auflistung mit IconImages als Icon-Datei unter einem angegebnen Pfad
    ''' </summary>
    Private Sub SaveImages(ByVal fileName As String, ByVal Images As List(Of IconImage))
        ' Neuen FileStream zum Speichern der Datei unter angegebenem Pfad erzeugen
        Dim fs As New FileStream(fileName, FileMode.Create, FileAccess.ReadWrite)

        Try
            ' BinaryWriter zum Schreiben der Daten in den Stream
            Dim Writer As New IO.BinaryWriter(fs)

            ' IconHeader schreiben
            Dim Head As New IconHeader(Images.Count)
            Head.Write(Writer)

            Dim ImageOffset As Integer = 6 + Images.Count * 16

            ' Fr jedes IconImage in der bergebenen IconImage-Liste
            For Each Img As IconImage In Images
                ' ImageOffset im IconDirectory aktualisieren
                Img.Directory.ImageOffset = ImageOffset
                ' IconDirectory schreiben
                Img.Directory.Write(Writer)
                ImageOffset += Img.Directory.BytesInRes
            Next

            ' Fr jedes IconImage aus der bergebenen IconImage-Liste
            ' die Bilddaten schreiben
            For Each Img As IconImage In Images
                Writer.Write(Img.ImageData)
            Next

        Finally
            ' Falls notwendig, FileStream frei geben
            If fs IsNot Nothing Then
                fs.Close()
            End If
        End Try
    End Sub

    ''' <summary>
    ''' erstellt ein neues IconImage-Objekt in gewnschtem IconImageFormat 
    ''' auf Grundlage einer angegebenen Bitmap und gibt den temporren
    ''' Speicherpfad dieses IconImage zurck
    ''' </summary>
    ''' <param name="Format">gewnschtes IconImageFormat</param>
    ''' <param name="BaseImage">als Grundlage verwendete Bitmap</param>
    ''' <returns>IconImage</returns>
    Public Overloads Function Add(ByVal Format As IconImageFormat, ByVal BaseImage As Bitmap) As IconImage
        Dim NewImage As IconImage = IconImage.Create(Format, BaseImage)
        m_Images.Add(NewImage)
        Return NewImage
    End Function

    ''' <summary>
    ''' fgt ein IconImage-Objekt zur IconImageList dieser Datei hinzu
    ''' </summary>
    ''' <param name="Image">IconImage</param>
    Public Overloads Sub Add(ByVal Image As IconImage)
        m_Images.Add(Image)
    End Sub

    ''' <summary>
    ''' entfernt ein IconImage-Objekt aus der IconImageList dieser Datei
    ''' </summary>
    Public Sub Remove(ByVal Image As IconImage)
        m_Images.Remove(Image)
        ' Aufgabe dieser Methode ist, die zu entfernende IconImage-Instanz
        ' temporr zu speichern, um sie spter wiederherstellen zu knnen.
        ' Also wird wenn notwendig, der eindeutige temporre Dateiname dieses
        ' IconImageObjekts zu den bekannten Dateinamen hinzu gefgt und 
        ' dieses IconImage in Form eines Icons im Temp-Ordner des Dateisystems gespeichert.
        ' Beim Disposing der Datei werden brigens alle temporren Dateien wieder glscht.
        If Not m_PersistedImages.Contains(Image.TempName) Then
            Dim Images As New List(Of IconImage)
            Images.Add(Image)
            SaveImages(Image.TempName, Images)
            m_PersistedImages.Add(Image.TempName)
        End If
    End Sub

    ''' <summary>
    ''' Erstellt ein IconImage in gewnschtem IconImageFormat
    ''' mit Undo-Redo-Funtionalitt
    ''' </summary>
    ''' <param name="Zielformat">gewnschtes IconImageFormat</param>
    ''' <param name="NewImagePath">wenn vorhanden, Pfad zur als Basis verwendeten Bilddatei, 
    ''' wird hier ein leerer Pfad angegeben, wird SelectedItem aus der IconImageList als Bild-Basis verwendet</param>
    Public Sub RunCreateNew(ByVal Zielformat As IconImageFormat, ByVal NewImagePath As String)
        Dim ExistingImage As IconImage = Nothing

        ' wenn unter den aktuell in der IconImageList vorhandenen IconImages eine Instanz
        ' im neu zu erstellenden IconImageFormat existiert, wird diese durch das neu zu erstellende
        ' IconImage-Objekt ersetzt. Um dieses Ersetzen rckgngig machen zu knnen,
        ' muss dieses Objekt fr den UndoManager persistent gemacht werden
        For Each img As IconImage In m_Images
            If img.CompareTo(Zielformat) = 0 Then
                ExistingImage = img
                Exit For
            End If
        Next


        If NewImagePath = "" Then
            ' Wenn kein Pfad fr eine Bilddatei angegeben wurde, wird das aktuell selektierte
            ' IconImage in der IconImageList als Bild-Basis verwendet
            ' Undo-Redo-Kommando zum Anfgen oder Ersetzen eines IconImage durch einen auf Basis des selektierten
            ' IconImage erstellten Images im gewnschten Format
            Dim Cmd As New CreateImageCloneCommand(Me, ExistingImage, Zielformat, m_Images.SelectedItem)
            ' Kommando zum UndoManager hinzufgen und ausfhren
            m_UndoManager.Push(Cmd)
        Else
            Try
                ' Wenn ein Pfad zu einer Bilddatei im Dateisystem angegeben wurde, erst mal 
                ' prfen, ob diese Bilddatei fehlerfrei geffnet werden kann
                Dim NewImage As Image = Image.FromFile(NewImagePath)
                NewImage.Dispose()
                ' Dann Undo-Redo-Kommando zum Anfgen oder Ersetzen eines IconImage durch einen 
                ' (auf Basis einer Bilddatei) erstellten IconImages im gewnschten Format erstellen
                Dim Cmd As New CreateNewImageCommand(Me, ExistingImage, Zielformat, NewImagePath)
                ' Kommando zum UndoManager hinzufgen und ausfhren
                m_UndoManager.Push(Cmd)
            Catch ex As Exception

            End Try

        End If

    End Sub

    ''' <summary>
    ''' Entfernt das in der IconImageList selektierte IconImage aus der Datei
    ''' mit Undo-Redo-Funtionalitt
    ''' </summary>
    Public Sub RunRemove()
        ' Undo-Redo-Kommando zum Lschen des aktuell selektierten iconImage erstellen
        Dim Cmd As New RemoveImageCommand(Me, m_Images.SelectedItem.TempName)
        ' Kommando zum UndoManager hinzufgen und ausfhren
        m_UndoManager.Push(Cmd)
    End Sub

    ''' <summary>
    ''' berschreibt die Dispose-Methode der Datei-Basisklasse
    ''' </summary>
    Protected Overrides Sub Dispose(ByVal disposing As Boolean)
        MyBase.Dispose(disposing)
        ' hier werden alle gegebenenfalls bei Undo/Redo erstellten temporren Dateien wieder gelscht
        For Each txt As String In m_PersistedImages
            If IO.File.Exists(txt) Then
                IO.File.Delete(txt)
            End If
        Next
    End Sub

    ''' <summary>
    ''' Erstellt eine Icon-Datei auf Grundlage einer vorhandenen Bilddatei in bevorzugten IconImage-Formaten
    ''' und gibt hierbei entfernte und neu erstellte IconImage-Objekte als AutoIconInfo zurck
    ''' </summary>
    Friend Function CreateAutoIcon(ByVal Path As String) As AutoIconInfo
        ' zunchst erst mal AutoIconInfo zur Rckgabe instanziieren
        Dim Result As New AutoIconInfo

        ' Beim Erstellen eines AutoIcon werden alle vorhandenen IconImages entfernt
        For i As Integer = m_Images.Count - 1 To 0 Step -1
            Dim Image As IconImage = m_Images(i)
            Result.RemovedImages.Add(Image.TempName)
            m_Images.Remove(Image)
            ' falls noch nicht geschehen, das zu entfernende IconImage fr Undo-Redo
            ' temporr speichern
            If Not m_PersistedImages.Contains(Image.TempName) Then
                Dim Images As New List(Of IconImage)
                Images.Add(Image)
                SaveImages(Image.TempName, Images)
                m_PersistedImages.Add(Image.TempName)
            End If
        Next

        ' Basis-Bild zum Erstellen der IconImages fr den Icon ist eine einzelne Bilddatei
        Dim BaseBitmap As Bitmap = Image.FromFile(Path)

        ' Fr alle bevorzugten Formate neue IconImages auf Grundlage des Basis-Bildes erstellen
        For Each Fmt As IconImageFormat In PreferredFormats()
            Result.CreatedImages.Add(Add(Fmt, BaseBitmap).TempName)
        Next

        m_Images.SelectedIndex = 0

        ' entfernte und neu erstellte IconImage-Objekte als AutoIconInfo zurck geben
        Return Result

    End Function

    ''' <summary>
    ''' liefert eine Liste mit bevorzugten IconImage-Formaten zum 
    ''' automatischen Erstellen von Icons mit mehreren IconImages
    ''' </summary>
    Private Function PreferredFormats() As List(Of IconImageFormat)
        Dim Result As New List(Of IconImageFormat)
        Result.Add(New IconImageFormat(48, 48, 32, False))
        Result.Add(New IconImageFormat(48, 48, 8, False))
        Result.Add(New IconImageFormat(48, 48, 4, False))

        Result.Add(New IconImageFormat(32, 32, 32, False))
        Result.Add(New IconImageFormat(32, 32, 8, False))
        Result.Add(New IconImageFormat(32, 32, 4, False))

        Result.Add(New IconImageFormat(16, 16, 32, False))
        Result.Add(New IconImageFormat(16, 16, 8, False))
        Result.Add(New IconImageFormat(16, 16, 4, False))

        Return Result
    End Function


    ''' <summary>
    ''' Erstellt eine Icon-Datei auf Grundlage einer vorhandenen Bilddatei in bevorzugten IconImage-Formaten
    ''' mit Undo-Redo-Funtionalitt
    ''' </summary>
    ''' <param name="Path">Pfad zur als Vorlage fr den Icon verwendeten Bilddatei</param>
    Public Sub RunCreateAutoIcon(ByVal Path As String)
        ' neues Kommando zum Erstellen eines AutoIcon erstellen
        Dim Cmd As New CreateAutoIconCommand(Me, Path)
        ' Kommando zum UndoManager hinzufgen und ausfhren
        m_UndoManager.Push(Cmd)
    End Sub

#End Region

#Region " Klassen "
    ''' <summary>
    ''' Klasse speziell fr das Erstellen eines AutoIcon
    ''' </summary>
    ''' <remarks>Enthlt temporre Pfade der neu erstellten und entfernten IconImages</remarks>
    Friend Class AutoIconInfo
        ' Liste mit temporren Pfaden neu erstellter IconImages
        Public CreatedImages As New List(Of String)
        ' Liste mit temporren Pfaden entfernter IconImages
        Public RemovedImages As New List(Of String)
    End Class
#End Region

End Class
