'  Copyright (c) 2006, Gustavo Franco
'  Email:  gustavo_franco@hotmail.com
'  All rights reserved.

'  Redistribution and use in source and binary forms, with or without modification, 
'  are permitted provided that the following conditions are met:

'  Redistributions of source code must retain the above copyright notice, 
'  this list of conditions and the following disclaimer. 
'  Redistributions in binary form must reproduce the above copyright notice, 
'  this list of conditions and the following disclaimer in the documentation 
'  and/or other materials provided with the distribution. 

'  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
'  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
'  PURPOSE. IT CAN BE DISTRIBUTED FREE OF CHARGE AS LONG AS THIS HEADER 
'  REMAINS UNCHANGED.

Imports System.io
''' <summary>
''' Basisklasse fr in einem Icon enthaltene Bilder
''' </summary>
Public MustInherit Class IconImage
    Inherits IconImageFormat

    ' Folgendes Problem besteht:
    ' Die einzelnen Icon-Images werden im Icon 
    ' entweder als Bitmap oder als PNG gespeichert.
    ' Und zwar tatschlich genaus so, wie sie eigentlich als Datei
    ' auf dem Desktop gespeichert werden.
    ' Fr die Nicht-PNG's gibt's doch eine Besonderheit: Hier werden zustzlich 
    ' And-Bytes gespeichert, diese enthalten Informationen, welche Pixel transparent sind.
    ' Die Bytes fr die einzelnen Bitmaps und PNG's werden einfach nacheinander 
    ' geschrieben.
    ' Zuvor kommt ein Icon-File-Header, der u.a. beschreibt, wie viele Einzel-Bitmaps 
    ' bzw. PNG's enthalten sind (FileHeader ist 6 bytes lang), danach nacheinander  
    ' fr jede enthaltene Bitmap oder PNG-Datei je ein Entry(Directory)-Eintrag (jeder 16 Bytes)
    ' der Infos zu Breite, Hhe, Farbanzahl, Bit-Farbtiefe der Einzel-Bitmaps oder -PNGs 
    ' enthlt (sowie Position und Lange der Einzel-Bitmaps bzw. PNG's innerhalb der Icon-Datei)
    ' An die Sequenz mit Entry-Eintrgen fgen sich dann die Daten der Bitmaps bzw. PNG's an.

    ' Im NET Framework kann ich eine Bitmap erstellen mit
    ' Dim BMP As New Bitmap (Breite, Hhe, Farbtiefe)
    ' mit BMP.Save(Filename, Imaging.ImageFormat.Png)
    ' wird diese Bitmap taschlich mit korrekter Farbtiefe gespeichert.
    ' aber beim erneuten Lesen so gespeicherter Bitmaps werden 8BPP Bilder als 32BPP ARGB 
    ' identifiziert.

    ' also kann ich fr ein PNG-Image beim ffnen des Icons nicht einfach nur Stream als Bitmap lesen
    ' und dann beim Speichern diese Bitmap wieder schreiben, sondern beim Speichern 
    ' muss ich eine neue Bitmap mit der gewnschten Farbtiefe erstellen und diese
    ' als PNG im Stream ablegen.

    ' Hier habe ich auch bercksichtigt, dass einmal vorhandene Original-Bilddaten und Entry-Daten 
    ' beim Speichern des Icons mglichst in Originalform beibehalten werden.

#Region " Private Felder "
    ' (diese Daten werden wie gelesen auch wieder geschrieben) (bis auf Positions-Angaben in den Directory-Daten)
    ' Icon-Directory-Daten 
    Protected m_Directory As IconDirectory
    ' Bilddaten 
    Protected m_ImageData() As Byte

    ' Bitmap, zum Anzeigen
    Protected m_BMP As Bitmap
    ' Dateiname fr temporre Dateien
    Private m_TempName As String
#End Region

#Region " Properties "
    ''' <summary>
    ''' ' Icon-Directory-Daten der Icon-Datei
    ''' </summary>
    ReadOnly Property Directory() As IconDirectory
        Get
            Return m_Directory
        End Get
    End Property

    ''' <summary>
    ''' Bilddaten
    ''' </summary>
    ReadOnly Property ImageData() As Byte()
        Get
            Return m_ImageData
        End Get
    End Property

    ''' <summary>
    ''' Dateiname zum temporren Speichern fr Undo/Redo
    ''' </summary>
    Property TempName() As String
        Get
            Return m_TempName
        End Get
        Set(ByVal value As String)
            m_TempName = value
        End Set
    End Property

    ''' <summary>
    ''' Bitmap zur Anzeige und als Quelle fr IconImages in anderen Formaten
    ''' </summary>
    ReadOnly Property Bitmap() As Bitmap
        Get
            Return m_BMP
        End Get
    End Property
#End Region

#Region " Konstruktoren "
    Public Sub New()
        ' Bei Neuerstellen einer abgeleiteten Klasse wird immer ein neuer temporrer Dateiname angelegt
        m_TempName = IO.Path.GetTempFileName
    End Sub
#End Region

#Region " Methoden "

    ''' <summary>
    ''' liefert ein neues IconImage fr eine Bitmap-Vorlage in festgelegtem Format 
    ''' </summary>
    ''' <param name="Format">IconImage-Format</param>
    ''' <param name="BaseImage">Bitmap-Vorlage</param>
    Public Shared Function Create(ByVal Format As IconImageFormat, ByVal BaseImage As Bitmap) As IconImage
        If Format.IsPNG Then
            Return New PNGImage(Format.Width, Format.BitsPerPixel, BaseImage, My.Settings.GifBackColor)
        Else
            Return New BitmapImage(Format.Width, Format.BitsPerPixel, BaseImage, My.Settings.GifBackColor)
        End If
    End Function

    ''' <summary>
    ''' liefert ein Item zum Anzeigen dieses IconImage in einem Listen-Control
    ''' </summary>
    ''' <param name="Selected">Information, ob dieses Item aktuell ausgewhlt ist</param>
    Public Function CreatePaintItem(ByVal Selected As Boolean) As Ctls.PaintItem
        Return New Ctls.ImageTextPaintItem(Get48Thumbnail(Me.Bitmap), MyBase.ToString, Selected)
    End Function

    ''' <summary>
    ''' liefert eine auf 48 Pixel verkleinerte 32-Bit-Bitmap
    ''' </summary>
    ''' <param name="Original">Original-Bitmap</param>
    Private Function Get48Thumbnail(ByVal Original As Image) As Bitmap

        ' neue 32 Bit Bitmap anlegen
        Dim NewImage As New Bitmap(48, 48, Imaging.PixelFormat.Format32bppArgb)
        ' Graphics-Objekt aus neuer Bitmap erzeugen
        Dim g As Graphics = Graphics.FromImage(NewImage)

        ' Rechteck-Begrenzung, in der die Original-Bitmap auf die 48-Pixel-Bitmap gezeichnet wird
        Dim Rect As Rectangle

        g.PixelOffsetMode = Drawing2D.PixelOffsetMode.HighQuality
        If Original.Width > 48 OrElse Original.Height > 48 Then
            ' Wenn das Original breiter oder hher als 48 Pixel ist, wird es 
            ' auf mglichest passende Gre verkleinert
            Rect = Ctls.Rect.GetMatchRect(48, 48, Original.Width, Original.Height)
            g.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBicubic
        Else
            ' wenn das Original kleiner als 48*48 Pixel ist, bleibt die Gre
            ' erhalten, es wird zentriert
            Rect = Ctls.Rect.GetCenterRect(48, 48, Original.Width, Original.Height)
            g.InterpolationMode = Drawing2D.InterpolationMode.NearestNeighbor
        End If

        ' Original im Graphics-Objekt im festgelegten Rechteck zeichnen
        g.DrawImage(Original, Rect)
        ' Graphics-Objekt frei geben
        g.Dispose()

        ' 48Pixel / 32 Bit-Bitmap zurckgeben
        Return NewImage
    End Function

#End Region

End Class
